from fastapi import APIRouter
from sources.api.v1.endpoints import hello_world

api_router = APIRouter()

api_router.include_router(
    hello_world.router,
    prefix='/hello-world',
    tags=['hello_world']
)



import os

import pytest

os.environ.setdefault('SETTINGS_MODULE', 'tests.conf')  # noqa

from fastapi_toolkit.db import (
    BaseModel,
    create_engine,
)
from tests.models import *  # noqa


@pytest.fixture
async def db():
    engine = create_engine()

    async with engine.begin() as connection:
        await connection.run_sync(BaseModel.metadata.create_all)

    yield

    async with engine.begin() as connection:
        await connection.run_sync(BaseModel.metadata.drop_all)

    await engine.dispose()




from fastapi import APIRouter
from sources.schemas.hello_world import HelloWorldModel

__all__ = (
    'router',
)


router = APIRouter()


@router.get(
    '/',
    name='Get hello world',
    response_model=HelloWorldModel,
)
async def get_hello_world():
    """Simple hello world response"""
    return {
        'project': '{{cookiecutter.project_name}}',
        'message': 'Hello world!'
    }



from typing import Any

from sqlalchemy.ext.declarative import (
    as_declarative,
    declared_attr,
)

__any__ = (
    'BaseModel',
)


@as_declarative()
class BaseModel:
    id: Any
    __name__: str

    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()



import asyncio
import os

from sqlalchemy.ext.asyncio import AsyncEngine

os.environ.setdefault('SETTINGS_MODULE', 'sources.conf')  # noqa

from logging.config import fileConfig

from alembic import context
from sources.models import *  # noqa
from sqlalchemy import (
    engine_from_config,
    pool,
)

from fastapi_toolkit.conf import settings

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
from fastapi_toolkit.db import BaseModel  # noqa

# target_metadata = mymodel.Base.metadata
target_metadata = BaseModel.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = settings.database_dsn
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection):

    context.configure(
        connection=connection,
        target_metadata=target_metadata,
    )

    with context.begin_transaction():
        context.run_migrations()


async def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    url = settings.database_dsn

    connectable = AsyncEngine(
        engine_from_config(
            configuration=dict(),
            url=url,
            prefix="sqlalchemy.",
            poolclass=pool.NullPool,
            future=True,
        )
    )

    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migrations)


if context.is_offline_mode():
    run_migrations_offline()
else:
    asyncio.run(run_migrations_online())



import argparse
import os

from cookiecutter.main import cookiecutter


def startproject(args):  # noqa
    dirname = os.path.dirname(os.path.abspath(__file__))
    cookiecutter(os.path.join(dirname, 'project_template'))


def main():
    parser = argparse.ArgumentParser(
        prog='fastapi-toolkit',
    )
    subparsers = parser.add_subparsers()
    parser_startproject = subparsers.add_parser(
        'startproject',
        help='Create new project'
    )
    parser_startproject.set_defaults(func=startproject)

    args = parser.parse_args()
    args.func(args)



from fastapi_toolkit.dependencies.filter import filter_by_fields
from fastapi_toolkit.dependencies.nginx import nginx_cache
from fastapi_toolkit.dependencies.order import order_by_fields
from fastapi_toolkit.dependencies.pagination import (
    BasePagination,
    LimitOffsetPagination,
    PageNumberPagination,
)

__all__ = (
    'filter_by_fields',
    'nginx_cache',
    'order_by_fields',
    'BasePagination',
    'PageNumberPagination',
    'LimitOffsetPagination'
)



from collections import namedtuple

from fastapi_toolkit.filters.func import FILTERS_LIST

filter_type = namedtuple(
    'FilterType',
    ('field', 'operators', 'cast_type', 'query_param_type')
)


def _noop(x):
    return x


MAPPING = {
    func.__name__.strip('_'): func
    for func in FILTERS_LIST
}


def filter_(
        field,
        operators: tuple,
        cast_type=None,
        query_param_type=None,
):
    return filter_type(
        field=field,
        operators={func.__name__.strip('_'): func for func in operators},
        cast_type=cast_type or _noop,
        query_param_type=query_param_type,
    )



from typing import (
    Type,
    Union,
)

from fastapi import (
    HTTPException,
    status as statuses,
)

from fastapi_toolkit.crud.base import (
    CRUDBase,
    ModelType,
)
from fastapi_toolkit.db.base_class import BaseModel

__all__ = (
    'get_object_or_404',
)


def _get_model_name(model_class: Union[CRUDBase, Type[BaseModel]]) -> str:
    if hasattr(model_class, 'model'):
        model_class = model_class.model
    return model_class.__table__.name


async def get_object_or_404(
        crud: CRUDBase,
        condition=None
) -> ModelType:
    obj = await crud.get(condition)
    if not obj:
        raise HTTPException(
            statuses.HTTP_404_NOT_FOUND,
            detail=f'Object does not exist in '
                   f'table {_get_model_name(crud.model)}'
        )
    return obj



from contextlib import asynccontextmanager
from typing import Optional

from sqlalchemy.ext.asyncio import (
    AsyncEngine,
    AsyncSession,
    create_async_engine,
)
from sqlalchemy.orm import sessionmaker

from fastapi_toolkit.conf import settings
from fastapi_toolkit.db.base_class import BaseModel

__all__ = (
    'create_session',
    'BaseModel',
    'init_db',
    'create_engine',
)


_engine: Optional[AsyncEngine] = None
_Session: Optional[sessionmaker] = None


def create_engine() -> AsyncEngine:
    global _engine

    if not _engine:
        _engine = create_async_engine(
            settings.database_dsn,
            pool_pre_ping=True
        )

    return _engine


def init_db():
    global _Session
    create_engine()

    if not _Session:
        _Session = sessionmaker(
            # autocommit=False,
            # autoflush=False,
            bind=_engine,
            expire_on_commit=False,
            class_=AsyncSession
        )


@asynccontextmanager
async def create_session():
    init_db()
    async with _Session() as session:  # noqa
        yield session



import abc

from fastapi import Query

__all__ = (
    'BasePagination',
    'PageNumberPagination',
    'LimitOffsetPagination',
)

from fastapi_toolkit.schemas.pagination import (
    PageNumberPagination as PageNumberPaginationSchema,
    LimitOffsetPagination as LimitOffsetPaginationSchema
)

class BasePagination:
    @property
    @abc.abstractmethod
    def database_params(self) -> dict:
        ...

    @property
    @abc.abstractmethod
    def params(self) -> dict:
        ...

    @property
    @abc.abstractmethod
    def schema(self):
        ...

    @staticmethod
    def _get_param(value: int, default: int = None):
        return value if value > 0 else default


class PageNumberPagination(BasePagination):
    def __init__(
            self,
            page: int = Query(1),
            page_size: int = Query(10)
    ):
        self.page = page
        self.page_size = page_size

    @property
    def database_params(self) -> dict:
        return {
            'offset': self._get_param((self.page - 1) * self.page_size, 0),
            'limit': self._get_param(self.page_size)
        }

    @property
    def params(self) -> dict:
        return {
            'page': self.page,
            'page_size': self.page_size
        }

    @property
    def schema(self):
        return PageNumberPaginationSchema


class LimitOffsetPagination(BasePagination):
    def __init__(
            self,
            offset: int = Query(0),
            limit: int = Query(10)
    ):
        self.offset = offset
        self.limit = limit

    @property
    def database_params(self):
        return {
            'offset': self._get_param(self.offset, 0),
            'limit': self._get_param(self.limit)
        }

    @property
    def params(self):
        return self.database_params

    @property
    def schema(self):
        return LimitOffsetPaginationSchema
